/************************************************************/
/* Fichier contenant differentes fonctions utilisees par la */
/* fonction de simplification, telles que l'application de  */
/* la methode des lignes de base de Mac Cluskey             */
/************************************************************/


#include <stdio.h>
#include "struct.h"
#include "fonct.h"
#include "incomp.h"
#include "pointeur.h"


/* Fonction retournant a partir des donnees apres regroupements */
/* une structure de type base * representant le nombre de fois  */
/* ou chaque minterm initial est utilise */

base * calcul(donne)
    cell * donne;
{
    cell * c;
    mint * m;
    base * b1, * b2;
    int deja;

    b1 = NULL;
    c = donne;
    while (c != NULL)
    {
	m = c->no;
	while (m != NULL)
	{
	    deja = 0;
	    if (b1 == NULL)
		b1 = new_base(m->numero);
	    else
	    {
		b2 = b1;
		while ((b2 != NULL) && (!deja))
		{
		    if (b2->min == m->numero)
		    {
			deja = 1;
			(b2->nb)++;
		    }
		    b2 = b2->suive;
		}

		if (!deja)
		{
		    b2 = b1;
		    while (b2->suive != NULL)
			b2 = b2->suive;
		    b2->suive = new_base(m->numero);
		}
	    }
	    m = m->next;
	}
	c = c->suiv;
    }

    return b1;
}

/* Renvoie 1 si le minterm numero n a une valeur non nulle dans cal */

int non_nul(n,cal)
	int n;
	base * cal;
{
	int ret, trouve = 0;

	while (!trouve)
	{
		if (cal->min == n)
		{
			trouve = 1;
			if (cal->nb == 0)
				ret = 0;
			else
				ret = 1;
		}
		else
			cal = cal->suive;
	}

	return ret;
}

/* Renvoie 1 s'il existe un minterm prenant une valeur non */
/* nulle dans cal (*p est la valeur eventuelle du premier  */
/* minterm de valeur non nulle dans cal)                   */

int existe_non_nul(cal,p)
	base * cal;
	int * p;
{
	int ret = 0;

	while ((cal != NULL) && (!ret))
	{
		if (cal->nb != 0)
		{
			ret = 1;
			*p = cal->min;
		}
		else
			cal = cal->suive;
	}

	return ret;
}


/* Renvoie le nombre de minterms de mi ayant une valeur */
/* non nulle dans cal */

int compte(mi, cal, t)
    mint * mi;
    base * cal;
    int * t;
{
    int n = 0;

    *t = 0;
    while (mi != NULL)
    {
	(*t)++;
	if (non_nul(mi->numero,cal))
	    n++;
	mi = mi->next;
    }

    return n;
}


/* Ajoute dans res tous les termes necessaires pour ne pas perdre  */
/* d'informations en les choisissant dans donne en ne retenant que */
/* les termes non redondants */

cell * mac_clusk(donne,res,cal)
	cell * donne;
	cell * res;
	base * cal;
{
	int num, max, tot, n, fin, va, ma, t;
	cell * c, * ce;
	mint * m;
	base * b;

	/* tant qu'il existe un numero de minterm non encore retenu */
	while (existe_non_nul(cal,&num))
	{
		c = donne;
		max = 0;
		tot = 0;
		while (c != NULL)
		{
		    /* recherche du terme contenant ce numero de minterm
		       et un maximum de numeros de minterms non encore
		       retenus */
			m = c->no;
			fin = 0;
			while ((m != NULL) && (!fin))
			{
				if (m->numero == num)
					fin = 1;
				else
					m = m->next;
			}
			if (fin)
			{
				m = c->no;
				n = compte(m,cal,&t);
				if ((n > max) || ((n == max) && (t > tot)))
				{
					max = n;
					tot = t;
					va = c->valeur;
					ma = c->masque;
				}
			}
			c = c->suiv;
		}

		c = donne;
		fin = 0;
		while (!fin)
		{
			if ((c->valeur == va) && (c->masque == ma))
				fin = 1;
			else
				c = c->suiv;
		}

		ce = res;
		while (ce->suiv != NULL)
			ce = ce->suiv;
		ce->suiv = new_cell(va,ma,c->no);
		ce = ce->suiv;

		m = ce->no;
		while (m != NULL)
		{
		    b = cal;
		    fin = 0;
		    while (!fin)
		    {
			if (b->min == m->numero)
			{
			    fin = 1;
			    /* mise de chaque numero de minterm
			       retenu a 0 dans cal */
			    b->nb = 0;
			}
			else
			    b = b->suive;
		    }
		    m = m->next;
		}
	}
	return res;
}


/* Fonction supprimant les termes redondants par la */
/* methode des lignes de base */

cell * ligne_base(donne,incomp)
    cell * donne;
    mint * incomp;
{
    base * cal, * bas, * b;
    cell * res, * temp, * cher;
    int trouve, max, va, ma, i;
    mint * m;

    /* remplissage de la structure cal (nb de fois ou
       chaque minterm est utilise) */
    cal = calcul(donne);

    /* met la valeur de tous les termes phi choisis egaux a 1
       a 0 dans cal de maniere a ce qu'ils n'influencent pas
       le choix d'un terme par rapport a un autre */
    bas = cal;
    while (bas != NULL)
    {
	if (appart(bas->min,incomp))
	    bas->nb = 0;
	bas = bas->suive;
    }

    res = NULL;
    bas = cal;
    while (bas != NULL)
    {
	/* si le minterm bas->min n'a ete utilise qu'une fois,
	   alors retenir dans la solution finale le terme
	   qui l'utilise */
	if (bas->nb == 1)
	{
		cher = donne;
		trouve = 0;
		while (!trouve)
		{
			m = cher->no;
			while ((m != NULL) && (!trouve))
			{
				if (m->numero == bas->min)
					trouve = 1;
				else
					m = m->next;
			}
			if (!trouve)
				cher = cher->suiv;
		}

		if (res == NULL)
		{
			res = new_cell(cher->valeur,cher->masque,cher->no);

			m = res->no;
			while (m != NULL)
			{
				b = cal;
				while (b->min != m->numero)
					b = b->suive;
				b->nb = 0;
				m = m->next;
			}
			temp = res;
		}
		else
		{
			temp->suiv = new_cell(cher->valeur,cher->masque,
					      cher->no);
			temp = temp->suiv;

			m = temp->no;
			while (m != NULL)
			{
				b = cal;
                                while (b->min != m->numero)
					b = b->suive;
				b->nb = 0;
				m = m->next;
			}

		}
	}
	bas = bas->suive;
    }

    /* cas ou aucun minterm n'a ete utilise qu'une seule fois : on
       ne retient pour l'instant que le terme utilisant le maximum
       de minterms */
    if (res == NULL)
    {
	temp = donne;
	max = 0;
	while (temp != NULL)
	{
	    trouve = compte(temp->no,cal,&i);
	    if (trouve > max)
	    {
		max = trouve;
		va = temp->valeur;
		ma = temp->masque;
	    }
	    temp = temp->suiv;
	}

	temp = donne;
	while ((temp->valeur != va) || (temp->masque != ma))
	    temp = temp->suiv;

	res = new_cell(va,ma,temp->no);

	m = temp->no;
	while (m != NULL)
	{
	    b = cal;
	    while (b->min != m->numero)
		b = b->suive;
	    b->nb = 0;
	    m = m->next;
	}
    }

    /* ajout des termes contenant des numeros de minterms non
       encore retenus */
    res = mac_clusk(donne,res,cal);

    del_base(cal);
    del_cell(donne);

    return res;
}


/* renvoie 1 si la combinaison des minterms de mi n'a pas deja */
/* ete retenue dans cel1 */

int pas_utilise(mi,cel1)
    mint * mi;
    cell * cel1;
{
    mint * m;
    cell * c;
    int ret = 0;

    while ((mi != NULL) && (!ret))
    {
	ret = 1;
	c = cel1;
	while ((c != NULL) && (ret))
	{
	    m = c->no;
	    while ((m != NULL) && (ret))
	    {
		if (m->numero == mi->numero)
		    ret = 0;
		else
		    m = m->next;
	    }
	    c = c->suiv;
	}

	mi = mi->next;
    }
    return ret;
}


int non_utilise(mi,en)
    mint * mi;
    ens * en;
{
    int ret = 1;

    while ((en != NULL) && (ret))
    {
	ret = pas_utilise(mi,en->element);
	en = en->autre;
    }

    return ret;
}


/* renvoie 1 si le terme de valeur v et de masque m n'appartient */
/* pas encore a cel */

int existe_pas(v,m,cel)
    int v,m;
    cell * cel;
{
    int ret = 1;

    while ((cel != NULL) && (ret))
    {
	if ((cel->valeur == v) && (cel->masque == m))
	    ret = 0;
	cel = cel->suiv;
    }
    return ret;
}
