/******************************************************/
/* Fichier contenant les fonctions d'allocation et de */
/* destruction des differentes structures de donnees  */
/******************************************************/


#include <stdio.h>
#include "struct.h"
#include <stdlib.h>


/* Fonctions d'allocations simples */

ens * mal_ens()
{
    ens * en = (ens *) malloc(sizeof(ens));
    return en;
}

cell * mal_cell()
{
    cell * cel = (cell *) malloc(sizeof(cell));
    return cel;
}


/* Fonctions de destructions de structures entieres */

void del_mint(min)
	mint * min;
{
	mint * m1, * m2;

	m1 = min;
	while (m1 != NULL)
	{
		m2 = m1->next;
		free(m1);
		m1 = m2;
	}
}


void del_ens(en)
    ens * en;
{
    ens * e;
    cell * c1, * c2;

    while (en != NULL)
    {
	c1 = en->element;
	while (c1 != NULL)
	{
	    del_mint(c1->no);
	    c2 = c1->suiv;
	    free(c1);
	    c1 = c2;
	}
	e = en->autre;
	free(en);
	en = e;
    }
}


void del_cell(cel)
    cell * cel;
{
    cell * c;
    mint * m1, * m2;

    while (cel != NULL)
    {
	m1 = cel->no;
	while (m1 != NULL)
	{
	    m2 = m1->next;
	    free(m1);
	    m1 = m2;
	}
	c = cel->suiv;
	free(cel);
	cel = c;
    }
}


void del_base(bas)
	base * bas;
{
	base * b;

	while (bas != NULL)
	{
		b = bas->suive;
		free(bas);
		bas = b;
	}
}


void del_varia(v)
    varia * v;
{
    varia * va;

    while (v != NULL)
    {
	va = v->suiv;
	free(v->var);
	free(v);
	v = va;
    }
}


void del_bin(b)
    bin * b;
{
    bin * bi;

    while (b != NULL)
    {
	bi = b->suiv;
	free(b);
	b = bi;
    }
}


void del_list_bin(l)
    list_bin * l;
{
    list_bin * li;

    while (l != NULL)
    {
	li = l->suiv;
	del_bin(l->terme);
	free(l);
	l = li;
    }
}


void del_poids_var(p)
    poids_var * p;
{
    poids_var * po;

    while (p != NULL)
    {
	po = p->suiv;
	free(p);
	p = po;
    }
}


void del_tab_var(t)
    tab_var * t;
{
    tab_var * ta;

    while (t != NULL)
    {
	ta = t->suiv;
	del_poids_var(t->terme);
	free(t);
	t = ta;
    }
}


void del_trans(t)
    trans * t;
{
    trans * tr;

    while (t != NULL)
    {
	tr = t->suiv;
	del_mint(t->comp);
	del_mint(t->incomp);
	del_mint(t->comp1);
	del_mint(t->incomp1);
	free(t);
	t = tr;
    }
}


/* Fonctions d'allocations et de remplissage des structures */

mint * new_mint(no)
    int no;
{
    mint * nouv = (mint *) malloc(sizeof(mint));

    nouv->numero = no;
    nouv->next = NULL;

    return nouv;
}


cell * new_cell(val,masq,m)
    int val;
    int masq;
    mint * m;
{
    mint * t;
    cell * part = (cell *) malloc(sizeof(cell));

    part->valeur = val;
    part->masque = masq;
    part->suiv = NULL;
    part->no = new_mint(m->numero);
    m = m->next;
    t = part->no;
    while (m != NULL)
    {
	t->next = new_mint(m->numero);
	t = t->next;
	m = m->next;
    }

    return part;
}


base * new_base(n)
    int n;
{
    base * b = (base *) malloc(sizeof(base));

    b->min = n;
    b->nb = 1;
    b->suive = NULL;

    return b;
}

varia * new_varia(chaine)
    char * chaine;
{
    varia * new = (varia*) malloc(sizeof(varia));

    new->var = (char*) malloc(8);
    strcpy(new->var,chaine);
    new->suiv = NULL;

    return new;
}


bin * new_bin(poids,b)
    int poids, b;
{
    bin * new = (bin*) malloc(sizeof(bin));

    new->poids = poids;
    new->bin = b;
    new->suiv = NULL;

    return new;
}


list_bin * new_list_bin(n)
    int n;
{
    list_bin * new = (list_bin*) malloc(sizeof(list_bin));

    new->valeur = n;
    new->terme = NULL;
    new->suiv = NULL;

    return new;
}


trans * new_trans(v)
    int v;
{
    trans * tr = (trans *) malloc(sizeof(trans));

    tr->var = v;
    tr->comp = NULL;
    tr->incomp = NULL;
    tr->comp1 = NULL;
    tr->incomp1 = NULL;
    tr->suiv = NULL;

    return tr;
}
