/**************************************************/
/* Fichier de traitement des circuits sequentiels */
/**************************************************/


#include <stdio.h>
#include <string.h>
#include "struct.h"
#include "affichage.h"
#include "pointeur.h"
#include "fonct.h"
#include "sequent.h"
#include "simpl.h"
#include "incomp.h"
#include "liste.h"
#include "entree.h"
#include "scrut.h"


void seq()
{
    list_bin * l;
    mint * m, * mi, * min, * minc, * mt;
    int n, j, in, ex, choix, fin;
    trans * t, * tr, * temp;
    cell * c;
    varia * list_var;

    printf("\nNombre de variables internes ? ");
    in=scrut();
    printf("Nombre de variables externes (entrees) ? ");
    ex=scrut();
    printf("\n");

    /* n = nb de variables total */
    n = in + ex;

    if ((list_var = variable(2,n)) == NULL)
    /* generation de noms de variables automatiques */
	list_var = cree_liste(ex,in);

    /* entree des etats et des transitions, puis traitement */
    l = sequent(in,ex,list_var);

    fin = 0;
    while (!fin)
    {
	t = expres(l,in,ex);

	printf("\nQuelle bascule ?\n");
	printf("  1 - bascule D\n");
	printf("  2 - bascule JK\n");
	printf("  3 - bascule T\n");
	printf("  4 - sortie\n");
	printf("\n\tChoix ? ");
	choix=scrut();
	printf("\n");

	if (choix == 4)
	    fin = 1;
	else
	{
	    if (choix != 1)
	    /* si ce ne sont pas des bascules D, decomposition des
	       fonctions sous la forme Qn+1 = f1.Q + f2./Q */
		t = decomp(t);

	    printf("Equations des bascules :\n\n");

	    tr = t;

	    j = 96 + in;
	    switch(choix)
	    {
	      case 1 :  	/* bascules D */

		while (tr != NULL)
		{
		    /* simplification des fonctions pour chaque variable */

		    if (tr->comp == NULL)
			c = NULL;
		    else
		    {
			if (tr->incomp != NULL)
			    c = prepare(n,&(tr->comp),tr->incomp);
			else
			    c = simplifie(n,tr->comp,NULL);
		    }

		    printf("D(%c) = ",j);
		    affiche(2,n,1,c,tr->comp,tr->incomp,list_var);
		    printf("\n\n");
		    del_cell(c);

		    j--;
		    tr = tr->suiv;
		}
		break;

	      case 2:         /* bascules JK */

		while (tr != NULL)
		{
		    /* traitements pour chaque variable */

		    /* traitement et simplification de la fonction f2 */

		    if (tr->comp1 == NULL)
			c = NULL;
		    else
		    {
			if (tr->incomp1 != NULL)
			    c = prepare(n,&(tr->comp1),tr->incomp1);
			else
			    c = simplifie(n,tr->comp1,NULL);
		    }
		    printf("J(%c) = ",j);
		    affiche(2,n,1,c,tr->comp1,tr->incomp1,list_var);
		    printf("\n");
		    del_cell(c);

		    /* traitement et simplification de la fonction /f1 */

		    if ((tr->incomp != NULL) && (tr->comp != NULL))
		    {
			c = prepare(n,&(tr->comp),tr->incomp);
			del_cell(c);
		    }

		    tr->comp = complement(tr->comp,n);
		    if (tr->comp == NULL)
			c = NULL;
		    else
			c = simplifie(n,tr->comp,tr->incomp);
		    tr->comp = complement(tr->comp,n);

		    printf("K(%c) = ",j);
		    affiche(2,n,1,c,tr->comp,tr->incomp,list_var);
		    printf("\n\n");
		    del_cell(c);

		    j--;
		    tr = tr->suiv;
		}
		break;

	      case 3:          /* bascules T */

		while (tr != NULL)
		{
		    /* traitement pour chaque variable */

		    /* traitement de f2./Q */

		    mi = tr->comp1;
		    m = NULL;
		    while (mi != NULL)
		    {
			if (!(mi->numero & puis(tr->var)))
			{
			/* on ne retient que les termes qui contiennent /Q */
			    if (m == NULL)
			    {
				m = new_mint(mi->numero);
				min = m;
			    }
			    else
			    {
				min->next = new_mint(mi->numero);
				min = min->next;
			    }
			}
			mi = mi->next;
		    }

		    mi = tr->incomp;
		    minc = NULL;
		    while (mi != NULL)
		    {
			if ((mi->numero & puis(tr->var)))
			{
			/* on ne retient que les termes qui contiennent Q */
			    if (minc == NULL)
			    {
				minc = new_mint(mi->numero);
				mt = minc;
			    }
			    else
			    {
				mt->next = new_mint(mi->numero);
				mt = mt->next;
			    }
			}
			mi = mi->next;
		    }

		    /* traitement de /f1.Q */

		    mi = tr->comp;
		    tr->comp = comple(mi,n,minc);
		    mi = tr->comp;
		    while (mi != NULL)
		    {
			if (mi->numero & puis(tr->var))
			{
			/* on ne retient que les termes qui contiennent Q */

			    if (m == NULL)
			    {
				m = new_mint(mi->numero);
				min = m;
			    }
			    else
			    {
				min->next = new_mint(mi->numero);
				min = min->next;
			    }
			}
			mi = mi->next;
		    }

		    mi = tr->incomp1;
		    while (mi != NULL)
		    {
			if (!(mi->numero & puis(tr->var)))
			{
			/* on ne retient que les termes qui contiennent /Q */
			    if (minc == NULL)
			    {
				minc = new_mint(mi->numero);
				mt = minc;
			    }
			    else
			    {
				mt->next = new_mint(mi->numero);
				mt = mt->next;
			    }
			}
			mi = mi->next;
		    }

		    /* simplification de la fonction obtenue f2./Q + /f1.Q */
		    if (m == NULL)
			c = NULL;
		    else
                    {
			if (minc != NULL)
			    c = prepare(n,&m,minc);
			else
			    c = simplifie(n,m,minc);
		    }

		    printf("T(%c) = ",j);
		    affiche(2,n,1,c,m,minc,list_var);
		    printf("\n\n");
		    del_cell(c);

		    j--;
		    tr = tr->suiv;
		    del_mint(m);
		    del_mint(minc);
		}
		break;
	    }
	}
	del_trans(t);
    }

    del_list_bin(l);
    del_varia(list_var);
}
