/********************************************************************/
/* Fichier contenant les fonctions d'initialisation de la structure */
/* principale et de simplification d'une fonction logique           */
/********************************************************************/


#include <stdio.h>
#include <math.h>
#include "struct.h"
#include "simpl.h"
#include "pointeur.h"
#include "fonct.h"


/* Fonction d'initialisation de la structure ens initiale a partir */
/* du nombre n de variables et des minterms de la fonction */

ens * initial(n,reunion)
    int n;
    mint * reunion;
{
    int i, nbr, masque, prem;
    mint * ent, * min;
    cell * ens_i;
    cell * part;
    ens * valeurs = (ens *) malloc(sizeof(ens));
    ens * temp;

    prem = 1;

    /* initialisation des ensembles en fonction du nombre de zero */

    for (i=0; i<=n; i++)
    {
	part = NULL;
	ent = reunion;

	ens_i = mal_cell();
	ens_i->suiv = NULL;

	part = ens_i;

	while (ent != NULL)
	{
	    nbr = 0;
	    masque = 1;

	    /* calcul du nombre nbr de 1 dans l'ecriture
	       binaire de ent->numero */

	    while (masque <= ent->numero)
	    {
		if (ent->numero & masque)
		    nbr ++;
		masque = masque << 1;
	    }
	    if (nbr == i)
	    {
		min = new_mint(ent->numero);
		part->suiv = new_cell(ent->numero, 0, min);
		del_mint(min);
		part = part->suiv;
	    }
	    ent = ent->next;

	}

	if (ens_i->suiv != NULL)
	{
	    /* il existe bien un numero de minterm dont l'ecriture
	       binaire est composee de i 1 */

	    if (prem)
	    {
		prem = 0;
		valeurs->element = ens_i->suiv;
		valeurs->autre = NULL;
		temp = valeurs;
	    }
	    else
	    {
		temp->autre = mal_ens();
		temp = temp->autre;
		temp->element = ens_i->suiv;
		temp->autre = NULL;
	    }
	}
	free(ens_i);
    }

    return valeurs;
}


/* Fonction principale de simplification de la fonction logique */
/* de n variables et dont les minterms sont contenus dans la    */
/* structure reunion de type mint * */

cell * simplifie(n,reunion,incomp)
    int n;
    mint * reunion;
    mint * incomp;
{
    ens * valeurs, * ens1, * ens2;
    cell * part, * partie, * cell1, * cell2, * autre, * aut;
    int ou_excl, nbr, masque, chgt = 1;
    mint * min1, * min2, * temp;

    autre = NULL;

    /* initialisation des structures */
    valeurs = initial(n,reunion);

    /* tant que l'on a pu faire des regroupements */
    while (chgt == 1)
    {
	ens1 = valeurs;
	chgt = 0;
	while (ens1->autre != NULL)
	{
	    partie = mal_cell();
	    partie->suiv = NULL;
	    part = partie;
	    ens2 = ens1->autre;
	    cell1 = ens1->element;

	    /* essai de regroupement de chaque terme de l'ensemble
	       i avec chaque terme de l'ensemble i+1 */

	    while (cell1 != NULL)
	    {
		cell2 = ens2->element;
		while (cell2 != NULL)
		{
		    if (cell1->masque == cell2->masque)
		    {
			/* si le masque est identique, on calcule le
			   nombre de positions binaires differentes
			   entre les deux numeros de minterms */

			ou_excl = cell1->valeur ^ cell2->valeur;
			nbr = 0;
			masque = 1;
			while ((masque <= ou_excl) && (nbr<=1))
			{
			    if (ou_excl & masque)
				nbr++;
			    masque = masque << 1;
			}

			/* s'il n'y a qu'une position differente et que ce
			   regroupement n'a pas deja ete effectue, alors
			   on le fait */

			if ((nbr == 1)&&(existe_pas(cell1->valeur&cell2->valeur,
						   cell1->masque | ou_excl,
						   partie->suiv)))
			{
			    chgt = 1; /* un regroupement est effectue */

			    part->suiv= new_cell(cell1->valeur & cell2->valeur,
						 cell1->masque | ou_excl,
						 cell1->no);
			    part = part->suiv;

			    min2 = part->no;
			    while (min2->next != NULL)
				min2 = min2->next;

			    /* concatenation de tous les numeros de minterms
			       utilises par ce regroupement */

			    temp = cell2->no;
			    while (temp != NULL)
			    {
				min1 = new_mint(temp->numero);
				min2->next = min1;
				min2 = min2->next;
				temp = temp->next;
			    }
			}
		    }
		    cell2 = cell2->suiv;
		}

		/* si le terme cell1 n'a pas donne lieu a un regroupement,
		   alors on le met dans une variable autre temporaire */

		if (pas_utilise(cell1->no,partie->suiv))
		{
		    if (autre == NULL)
		    {
			autre = new_cell(cell1->valeur,cell1->masque,cell1->no);
			aut = autre;
		    }
		    else
		    {
			aut->suiv = new_cell(cell1->valeur,cell1->masque,cell1->no);
			aut = aut->suiv;
		    }
		}
		cell1 = cell1->suiv;
	    }

	    if (partie->suiv != NULL)
	    {
		del_cell(ens1->element);
		ens1->element = partie->suiv;
	    }
	    free(partie);
	    ens1 = ens1->autre;
	}

	ens1 = valeurs;
        if (ens1->autre != NULL)
	{
	    ens1 = (ens1->autre)->autre;
	    ens2 = valeurs;
	    while (ens1 != NULL)
	    {
		ens1 = ens1->autre;
		ens2 = ens2->autre;
	    }

	    cell1 = NULL;
	    part = (ens2->autre)->element;
	    while (part != NULL)
	    {
		if (cell1 == NULL)
		{
		    cell1 = new_cell(part->valeur,part->masque,part->no);
		    cell2 = cell1;
		}
		else
		{
		    cell2->suiv = new_cell(part->valeur,part->masque,part->no);
		    cell2 = cell2->suiv;
		}
		part = part->suiv;
	    }

	    del_ens(ens2->autre);
	    ens2->autre = NULL;

	    cell2 = cell1;
	    while (cell2 != NULL)
	    {
		if (non_utilise(cell2->no,valeurs))
		{
		    if (autre == NULL)
		    {
			autre = new_cell(cell2->valeur,cell2->masque,cell2->no);
			aut = autre;
		    }
		    else
		    {
			aut->suiv = new_cell(cell2->valeur,cell2->masque,cell2->no);
			aut = aut->suiv;
		    }
		}
		cell2 = cell2->suiv;
	    }
	    del_cell(cell1);

	}
    }


    /* mise des resultats apres les differents regroupements
       sous forme d'un unique pointeur de type cell *    */

    cell1 = valeurs->element;
    part = new_cell(cell1->valeur, cell1->masque, cell1->no);
    aut = part;

    cell1 = cell1->suiv;
    while (cell1 != NULL)
    {
	aut->suiv = new_cell(cell1->valeur, cell1->masque, cell1->no);
	aut = aut->suiv;
	cell1 = cell1->suiv;
    }

    /* ajout des elements de la variable temporaire autre */
    cell1 = autre;
    while (cell1 != NULL)
    {
	aut->suiv = new_cell(cell1->valeur, cell1->masque, cell1->no);
	aut = aut->suiv;
	cell1 = cell1->suiv;
    }

    del_ens(valeurs);
    del_cell(autre);

    /* suppression des termes redondants par la methode
       des lignes de base  */

    part = ligne_base(part,incomp);

    return part;
}
